/* $Id: catenary.java,v 1.46 2013/02/22 00:04:14 mit Exp $ */
/* catenary.java v0.2 */

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.util.*;
import java.text.DecimalFormat;

public class catenary extends Applet
    implements ActionListener, AdjustmentListener, ItemListener,
	       MouseListener, MouseMotionListener, Runnable
{
    private static final long serialVersionUID = 1L;
    private final int KB = 88;
    private final int KC = KB/2-1;
    private final int OCTAVE = 12;
    private final int OCT2 = 24;
    private final int CENT = 1200;
    private final int PITCH = 440;
    private final int INHA = 0;
    private final int COSH = 1;
    private final int SINH = 2;
    private final int NAPI = 3; // Napier
    private final int BEATS = 4;
    private final int SMPL = 5;
    private final int SPEED = 9;
    private final double ML10 = Math.log(10);
    private final String[] Graph = 
    {"Inha.","Cosh","Sinh","-e","Beats","Samp."};
    private final String[] OCTAVE_NAME = 
    {"A2","A1","A","a","a1","a2","a3","a4"};
    final String[] KEY_NAME =
    {"A..","B..","H..","C..","Cis","D..","Es.","E..","F..","Fis","G..","Gis"};

    private Button btPlus;
    private Button btMinus;
    private Button btStart;
    private Button btReverse;
    private Button btAbs;
    private Button[] btGraph;
    private Label lGrade;
    private Label lA49i;
    private Label lWound;
    private Label lDiff;
    private Label lTreble;
    private Label lBass;
    private Label lPitch;
    private Label lWire;
    private Label lSinh;
    private Label lBeat;
    private Label lSample;
    private Scrollbar scrTreble;
    private Scrollbar scrBass;
    private Scrollbar scrPitch;
    private Scrollbar scrWire;
    private Scrollbar scrSinh;
    private Scrollbar scrBeat;
    private Scrollbar scrSample;
    private Choice chSpeed;
    private Choice chInterval;
    private TextField tfData;

    private Graphics gi;
    private Image ig;
    private Font Fnt;
    private Canvas can;
    private Panel PN, PE, PW, PS;
    private DecimalFormat df1;
    private DecimalFormat df2;
    private FontMetrics fm;
    private Thread thr;

    private boolean Msw;
    private boolean RUNsw;
    private boolean CALCsw;
    private boolean SMPLsw;
    private boolean REVsw;;
    private boolean ABSsw;
    private boolean DEBUG = false;
    private int FNT;
    private int W, H, H2, W2;
    private int Wound = 27; // 28-1
    private int MouseX;
    private int MouseY;
    private int MouseG;
    private int GRAPH;
    private int KEY;
    private int COSH_COUNTER;
    private int Stime;
    private int ITV2O;
    private double SHBE;
    private double BEAT;
    private double SAMPLE;
    private double MAXH = 64;
    private double MAXB = 64;
    private double Grade = 0.087;
    private double A49i = 0.55;
    private double[] Inha;
    private double[] Beat;
    private double[] Cosh;
    private double[] Sinh;
    private double[] Napi;
    private double[] Beats;
    private double[] SampC;
    private double[] SampS;

    private MakeInharmonicity MI;
    private Interval IN;

    public void init() {
	String fnt = getParameter("font");
        FNT = (fnt != null) ? Integer.parseInt(fnt) : 12;
        Fnt = new Font("SansSerif", Font.PLAIN, FNT);
        setFont(Fnt);
	fm = getFontMetrics(Fnt);
	df1 = new DecimalFormat("#0.0##");
	df2 = new DecimalFormat("#0.0#");

	IN = new Interval();
	buildLayout();
	init_seting();
    }
    public void buildLayout() {
        setBackground(Color.lightGray);
        setLayout(new BorderLayout());

	int num = Graph.length;
        PW = new Panel();
        PW.setLayout(new GridLayout(num,1,0,0));
        add(PW, "West");

	btGraph = new Button[num];
	for (int i = 0; i < num; i++) {
            btGraph[i] = new Button(Graph[i]);
            btGraph[i].addActionListener(this);
            PW.add(btGraph[i]);
        }
	//---

        PE = new Panel();
        PE.setLayout(new GridLayout(5,1,0,0));
        add(PE, "East");

	btPlus = new Button("+");
        btPlus.addActionListener(this);
        PE.add(btPlus);

	btMinus = new Button("-");
        btMinus.addActionListener(this);
        PE.add(btMinus);

	PE.add(new Label(""));

	btReverse = new Button("->");
        btReverse.addActionListener(this);
        PE.add(btReverse);

	btAbs = new Button("abs");
	btAbs.addActionListener(this);
	PE.add(btAbs);
	//---

        PN = new Panel();
        PN.setLayout(new FlowLayout());
	add(PN, "North");

	tfData = new TextField(26);
        tfData.setEditable(false);
        PN.add(tfData);

	chInterval = new Choice();
        for (int i = 0; i < IN.getLen(); i++)
            chInterval.addItem(IN.toString(i));
        PN.add(chInterval);
        chInterval.select(IN.getIinte(OCTAVE));
        chInterval.addItemListener(this);

	chSpeed = new Choice();
	for (int i = 0; i < SPEED; i++)
	    chSpeed.addItem(""+(i+1));
	PN.add(chSpeed);
        chSpeed.addItemListener(this);

	btStart = new Button("Start");
        btStart.addActionListener(this);
        PN.add(btStart);
	//---

        PS = new Panel();
        PS.setLayout(new GridLayout(4,5,0,0));
        add(PS, "South");

	scrTreble = new Scrollbar(Scrollbar.HORIZONTAL,87,10,0,130);
        PS.add(scrTreble);
        scrTreble.setBlockIncrement(10);
        scrTreble.addAdjustmentListener(this);

	lTreble = new Label("TrebleG:___",Label.LEFT);
        PS.add(lTreble);

	lDiff = new Label("[ diff:____ ]",Label.CENTER);
        PS.add(lDiff);

	lSinh = new Label("Sinh:___",Label.RIGHT);
        PS.add(lSinh);

	scrSinh = new Scrollbar(Scrollbar.HORIZONTAL,10,1,0,1001);
        PS.add(scrSinh);
        scrSinh.setBlockIncrement(10);
        scrSinh.addAdjustmentListener(this);
	//
        scrBass = new Scrollbar(Scrollbar.HORIZONTAL,60,10,0,130);
        PS.add(scrBass);
        scrBass.setBlockIncrement(10);
        scrBass.addAdjustmentListener(this);

	lBass = new Label("BassG:___",Label.LEFT);
        PS.add(lBass);

	lGrade = new Label("Grade:_____",Label.CENTER);
        PS.add(lGrade);

	lBeat = new Label("-e:___",Label.RIGHT);
	PS.add(lBeat);

	scrBeat = new Scrollbar(Scrollbar.HORIZONTAL,10,1,0,61);
        PS.add(scrBeat);
        scrBeat.setBlockIncrement(10);
        scrBeat.addAdjustmentListener(this);
	//
        scrPitch = new Scrollbar(Scrollbar.HORIZONTAL,55,10,1,100);
        PS.add(scrPitch);
        scrPitch.setBlockIncrement(10);
        scrPitch.addAdjustmentListener(this);

	lPitch = new Label("Pitch:___",Label.LEFT);
        PS.add(lPitch);

	lA49i = new Label("A49:_____",Label.CENTER);
        PS.add(lA49i);

	PS.add(new Label(""));
	PS.add(new Label(""));
	//
        scrWire = new Scrollbar(Scrollbar.HORIZONTAL,28,1,0,49); //49-1
        PS.add(scrWire);
        scrWire.setBlockIncrement(10);
        scrWire.addAdjustmentListener(this);

	lWire = new Label("WoundW:___",Label.LEFT);
        PS.add(lWire);

	lWound = new Label("Wound:_____",Label.CENTER);
        PS.add(lWound);

	lSample = new Label("Sample:___",Label.RIGHT);
	PS.add(lSample);

	scrSample = new Scrollbar(Scrollbar.HORIZONTAL,10,1,1,21);
        PS.add(scrSample);
        scrSample.setBlockIncrement(10);
        scrSample.addAdjustmentListener(this);
	//---

        can = new myCanvas();
        add(can, "Center");

        can.addMouseListener(this);
        can.addMouseMotionListener(this);
    }
    public void start() {
	W = getCanvasD().width;
        H = getCanvasD().height;

	can.setSize(W, H);
        ig = createImage(W, H);
        gi = ig.getGraphics();
        gi.setFont(Fnt);

	H2 = H/2;
	W2 = W/2;
    }
    public Dimension getCanvasD() {
        Dimension de = PE.getSize();
        Dimension dn = PN.getSize();
        Dimension ds = PS.getSize();
        Dimension dw = PW.getSize();
        Dimension da = getSize();
        int cw = da.width-(de.width+dw.width);
        int ch = da.height-(dn.height+ds.height)-8;
        Dimension can = new Dimension(cw, ch);

        return can;
    }
    public void init_seting() {
        MI = new MakeInharmonicity();
	Inha = MI.getInha();
	Beat = new double[KB];
	Cosh = new double[KB];
	Sinh = new double[KB];
	Napi = new double[KB];
	Beats = new double[KB];
	SampS = new double[KB];
	SampC = new double[KB];

	KEY = 37-1;
	GRAPH = INHA;
	ITV2O = IN.getIinte(OCT2); // 2octave#
        set_treble_grade();
	set_bass_grade();
        set_pitch_inha();
        set_wound_wire();
	set_beat();
	set_sinh();
	set_sample();

	set_enabled();
    }
    public void set_enabled() {
	boolean ssw = (GRAPH == SMPL);
	scrSample.setEnabled(ssw);
	lSample.setEnabled(ssw);
	btStart.setEnabled(!ssw);
	lSinh.setEnabled(!ssw);
	scrSinh.setEnabled(!ssw);
	lBeat.setEnabled(!ssw);
	scrBeat.setEnabled(!ssw);

	boolean bsw = (GRAPH == SINH || GRAPH == NAPI || GRAPH == BEATS);
	btAbs.setEnabled(bsw);
	btReverse.setEnabled(bsw);

	boolean isw = (GRAPH == INHA ||
		       (GRAPH == COSH && COSH_COUNTER != 0));
	btMinus.setEnabled(!isw);
	btPlus.setEnabled(!isw);
    }
    //
    public void run() {
	int stime = set_stime();
	int w0 = Wound;
	double a0 = A49i;
	double g0 = Grade;

	try {
	    while(RUNsw) {
		run_wound(stime);
		run_grade(stime);
		run_a49i(stime);

		if (DEBUG) {
		    System.out.println("A:"+dform(A49i)+" a0:"+dform(a0)+
				       " G:"+dform(Grade)+" g0:"+dform(g0)+
				       " W:"+Wound+" w0:"+w0);
		}

		if (w0 == Wound & a0 == A49i & g0 == Grade)
		    break;
		thr.sleep(stime);

		w0 = Wound;
		a0 = A49i;
		g0 = Grade;
	    }
	} catch (InterruptedException e) {
	    System.out.println(e);
	}

	set_go_button(true);
	thr = null;
	CALCsw = false;
	can.repaint();
    }
    public int set_stime() {
	int st = Integer.parseInt(chSpeed.getSelectedItem());
	return st*10; // 10~100
    }
    public double get_a_diff() {
	double sa = 0.002;
	double d0 = calc_inha(A49i);
	double dp = calc_inha(A49i+sa);
	double dm = calc_inha(A49i-sa);

	return Math.max(Math.abs(d0-dp), Math.abs(d0-dm));
    }
    public void run_a49i(int stime) {
       	double di, df = A49i;
	double ih = Math.abs(log_base(A49i)-log_base(Inha[48]));
	double sa = (A49i > 2.0) ? 0.5 : (ih > 1.4) ? 0.1 : 0.002;
	double d0 = calc_inha(A49i);
	double dp = calc_inha(A49i+sa);
	double dm = calc_inha(A49i-sa);
	if (d0 > dp)
	    sa = sa;
	else if (d0 >= dm)
	    sa = -sa;

	if (DEBUG) {
	    System.out.println("A:"+dform(A49i)+" d0:"+dform(d0)+
			       " dp:"+dform(dp)+"("+dform(A49i+sa)+")"+
			       " dm:"+dform(dm)+"("+dform(A49i-sa)+")");
	}

	try {
	    while(RUNsw) {
		df += sa;
		di = calc_inha(df);

		CALCsw = false;
		can.repaint();
		thr.sleep(stime);

		if (di < d0) {
		    A49i = df;
		    d0 = di;
		} else
		    break;
	    }
	} catch (InterruptedException e) {
	    System.out.println(e);
	}
    }
    public double calc_inha(double ih) {
        double[] keys = makeCosh(Wound, ih, Grade);
        double di = diff_h(keys);

	if (DEBUG)
	    System.out.println("Iih: "+dform(ih)+" di: "+dform(di));

        return di;
    }
    public double get_g_diff() {
	double sa = 0.001;
	double d0 = calc_grade(Grade);
	double dp = calc_grade(Grade+sa);
	double dm = calc_grade(Grade-sa);

	return Math.max(Math.abs(d0-dp), Math.abs(d0-dm));
    }
    public void run_grade(int stime) {
	double di, df;
	double sa = 0.001;
	double d0 = calc_grade(Grade);
	double dp = calc_grade(Grade+sa);
	double dm = calc_grade(Grade-sa);
	if (dp < d0)
	    sa = sa;
	else if (dm <= d0)
	    sa = -sa;

	if (DEBUG) {
	    System.out.println("G:"+dform(Grade)+" d0:"+dform(d0)+
			       " dp:"+dform(dp)+"("+dform(Grade+sa)+")"+
			       " dm:"+dform(dm)+"("+dform(Grade-sa)+")");
	}

	df = Grade;
	try {
	    while(RUNsw) {
		df += sa;
		di = calc_grade(df);

		CALCsw = false;
		can.repaint();
		thr.sleep(stime);

		if (di < d0) {
		    Grade = df;
		    d0 = di;
		} else
		    break;
	    }
	} catch (InterruptedException e) {
	    System.out.println(e);
	}
    }
    public double calc_grade(double gr) {
	double[] keys = makeCosh(Wound, A49i, gr);
        double di = diff_h(keys);

	if (DEBUG)
	    System.out.println("Ggr: "+dform(gr)+" di: "+dform(di));

        return di;
    }
    public double get_w_diff() {
	if (Wound < 1) {
	    Wound = 0;
	    return -1;
	}
	double d0 = calc_wound(Wound);
	double dp = calc_wound(Wound+1);
	double dm = calc_wound(Wound-1);

	return Math.max(Math.abs(d0-dp), Math.abs(d0-dm));
    }
    public void run_wound(int stime) {
	double di;
	int df, sa = 1;
	if (Wound < 1) {
	    Wound = 0; // 1
	} else if (Wound > 44)
	    Wound = 44;
	double d0 = calc_wound(Wound);
	double dp = calc_wound(Wound+sa);
	double dm = calc_wound(Wound-sa);
	if (dp < d0)
	    sa = sa;
	else if (dm <= d0)
	    sa = -sa;

	if (DEBUG) {
	    System.out.println("W:"+Wound+" d0:"+dform(d0)+
			       " dp:"+dform(dp)+" dm:"+dform(dm));
	}

	df = Wound;
	try {
	    while(RUNsw) {
		df += sa;
		di = calc_wound(df);

		CALCsw = false;
		can.repaint();
		thr.sleep(stime);

		if (di < d0) {
		    Wound = df;
		    d0 = di;
		}
		if (df < 0 || df > 48)
		    break;
	    }
	} catch (InterruptedException e) {
	    System.out.println(e);
	}
    }
    public double calc_wound(int ww) {
	double[] keys = makeCosh(ww, A49i, Grade);
        double di = diff_h(keys);

	if (DEBUG)
	    System.out.println("Www: "+ww+" di: "+dform(di));

        return di;
    }
    //    
    public void set_go() {
	if (thr == null) {
	    RUNsw = true;
	    Msw = false;
	    set_go_button(false);

	    thr = new Thread(this);
            thr.start();
        } else {
	    RUNsw = false;
	    set_go_button(true);

            thr = null;
	}
    }
    public void stop() {
        if (thr != null)
            thr = null;
    }
    public void set_go_button(boolean sw) {
        btStart.setLabel(sw ? "Start" : "Stop ");
    }
    //
    public void mouseMoved(MouseEvent me) {
	if (Msw) {
	    check_key(me.getX());

	    if (GRAPH == INHA) {
		A49i = Math.pow(10, (H2-me.getY())/Byh);
		Wound = (int)(me.getX()/DKEY);

		CALCsw = false;
	    }
	    can.repaint();
	}
    }
    public void check_key(int mx) {
	KEY = (int)(mx/DKEY);
	if (KEY < 0)
	    KEY = 0;
	else if (KEY >= KB)
	    KEY = KB-1;
    }
    public void mouseDragged(MouseEvent me) {
	if (GRAPH == INHA) {
	    Grade = 0.15*me.getX()/W;

	    CALCsw = false;
	    can.repaint();
	}
    }
    public void mousePressed(MouseEvent m) {}
    public void mouseClicked(MouseEvent m) {
	Msw = !Msw;
    }
    public void mouseReleased(MouseEvent m){}
    public void mouseEntered(MouseEvent m) {}
    public void mouseExited (MouseEvent m) {}
    public void actionPerformed(ActionEvent e) {
	for (int i = 0; i < Graph.length; i++) {
            if (e.getSource() == btGraph[i]) {
                GRAPH = i;
		if (GRAPH == COSH)
		    cosh_counter();
                break;
            }
        }
	if (e.getSource() == btStart)
	    set_go();

	else if (e.getSource() == btPlus)
	    set_height(true);
	else if (e.getSource() == btMinus)
	    set_height(false);

	else if (e.getSource() == btReverse)
	    set_reverse();

	else if (e.getSource() == btAbs)
	    set_abs();

	set_enabled();
	can.repaint();
    }
    public void set_abs() {
	ABSsw = !ABSsw;

	btAbs.setLabel(ABSsw ? "ABS" : "abs");
    }
    public void set_reverse() {
	REVsw = !REVsw;

	btReverse.setLabel(REVsw ? "<-" : "->");
	CALCsw = false;
    }
    public void cosh_counter() {
	COSH_COUNTER++;
	if (COSH_COUNTER > 1)
	    COSH_COUNTER = 0;
    }
    public void set_height(boolean sw) {
	if (GRAPH == BEATS) {
	    MAXB = (sw) ? MAXB*2 : MAXB/2;
	    if (MAXB < 8)
		MAXB = 8.0;
	} else {
	    MAXH = (sw) ? MAXH*2 : MAXH/2;
	    if (MAXH < 8)
		MAXH = 8.0;
	}
    }
    public void adjustmentValueChanged(AdjustmentEvent e) {
	if (e.getSource() == scrTreble)
            set_treble_grade();
	if (e.getSource() == scrBass)
            set_bass_grade();
	if (e.getSource() == scrPitch)
            set_pitch_inha();
	if (e.getSource() == scrWire)
            set_wound_wire();

	if (e.getSource() == scrBeat)
	    set_beat();
	if (e.getSource() == scrSinh)
	    set_sinh();

	if (e.getSource() == scrSample)
	    set_sample();

	CALCsw = false;
	can.repaint();
    }
    public void set_sample() {
	SAMPLE = scrSample.getValue()/10.0;
	lSample.setText("Sample:"+SAMPLE);
	SMPLsw = false;
    }
    public void set_sinh() {
	double be = scrSinh.getValue()/100.0;
	lSinh.setText("Sinh:"+be);
	SHBE = be;
    }
    public void set_beat() {
	double be = scrBeat.getValue()/10.0;
	lBeat.setText("-e:"+be);
	BEAT = be;
    }
    public void set_treble_grade() {
        double it = scrTreble.getValue()/1000.0;
        lTreble.setText("TrebleG:"+(float)it);
        MI.setTrebleGrade(it);
	Inha = MI.getInha();
    }
    public void set_bass_grade() {
        double ib = scrBass.getValue()/1000.0;
        lBass.setText("BassG:"+(float)ib);
        MI.setBassGrade(ib);
	Inha = MI.getInha();
    }
    public void set_pitch_inha() {
        double a49 = scrPitch.getValue()/100.0;
        lPitch.setText("Pitch:"+(float)a49);
        MI.setA49i(a49);
	Inha = MI.getInha();
    }
    public void set_wound_wire() {
        int ww = scrWire.getValue();
        lWire.setText("WoundW:"+ww);
	MI.setWoundWire(ww);
	Inha = MI.getInha();
    }
    public void itemStateChanged(ItemEvent e) {
	can.repaint();
    }
    //
     public double diff_h(double[] keys) {
	double df = 0;
	for (int i = 0; i < KB; i++)
	    df += Math.abs(calc_log(keys[i])-calc_log(Inha[i]));

	return df;
    }
    public double calc_log(double ih) {
	return log_base(ih);
    }
    public double[] makeCosh(int ww, double a49, double gr) {
        double[] tmp = new double[KB];

        double df = a49/get_cosh(48, ww, gr); // 49-1ͤȤ
        for (int i = 0; i < KB; i++)
            tmp[i] = get_cosh(i, ww, gr)*df;

        return tmp;
    }
    public void calcBeat() { // 2octave
	int[] itv = IN.getIdata(REVsw ? -ITV2O : ITV2O);

	Beat = calc_beat0(itv);
    }
    public double[] calc_beat0(int[] itv) {
	double[] be = new double[KB];
	int max = REVsw ? KB : KB-itv[2];
	int start = REVsw ? -itv[2] : 0;

	for (int i = start; i < max; i++)
	    be[i] = calc_beat(i, itv);

	return be;
    }
    public double calc_beat(int key, int[] v) { // 4,1,24...
	int kh = key+v[2];
	double fh = ctof(getFreq(kh)*v[1], get_icent(kh, v[1]));
	double fl = ctof(getFreq(key)*v[0], get_icent(key, v[0]));

	return fh-fl;
    }
    public void calcBeats(int num) { // chInterval(쥯#)
       	int[] itv = IN.getIdata(REVsw ? -num : num);

	Beats = calc_beat0(itv);
    }
    public double ctof(double fr, double ce) {
	return fr*Math.pow(2, ce/CENT);
    }
    public double getFreq(int key) {
	return PITCH*Math.pow(2,(key-48)/(double)OCTAVE);
    }
    public double get_icent(int key, int pr) {
        return Cosh[key]*pr*pr+(Sinh[key]-Napi[key]);
    }
    //
    public void write_tfdata() {
	StringBuffer sb = new StringBuffer(30);

	if (GRAPH == SMPL)
	    sb.append((KEY-44)+": ");
	else
	    sb.append(getKeyname(KEY)+": ");
	switch (GRAPH) {
        case INHA:
	    sb.append(dform(MI.getInha(KEY)));
	    sb.append(" [ "+dform(Cosh[KEY])+" ]");
	    break;
	case SINH:
	    sb.append(dform(Sinh[KEY]));
	    sb.append(get_beat());
	    sb.append(" <"+dform(get_sce())+">");
	    break;
	case COSH:
	    sb.append(dform(Cosh[KEY]));
	    if (COSH_COUNTER < 1)
		sb.append(" <"+dform(get_sce())+">");
	    break;
	case NAPI:
	    sb.append(dform(-Napi[KEY]));
	    sb.append(get_beat());
	    sb.append(" <"+dform(get_sce())+">");
	    break;
	case BEATS:
	    int num = IN.getIvalue(chInterval.getSelectedIndex());

	    if (REVsw)
		sb.append((KEY < num) ? "..." : dform(Beats[KEY]));
	    else
		sb.append((KEY < KB-num) ? dform(Beats[KEY]) : "___");
	    sb.append(" "+get_beat());
	    break;
	case SMPL:
	    sb.append(dform(SampC[KEY]));
	    sb.append(" : "+dform(SampS[KEY]));
	    break;
	default:
	}

	tfData.setText(sb.toString());
    }
    public double get_sce() {
	return Sinh[KEY]-Napi[KEY]+Cosh[KEY];
    }
    public String getKeyname(int key) {
        return (key+1)+"("+KEY_NAME[key%OCTAVE]+")";
    }
    public String get_beat() {
	if (REVsw)
	    return (KEY < OCT2) ?
		" ( ... )" : " ( "+dform(Beat[KEY])+" )";
	else
	    return (KEY < KB-OCT2) ?
		" ( "+dform(Beat[KEY])+" )" : " ( ___ )";
    }
    public void write_label(double df) {
	lGrade.setText("Grade:"+dform(Grade));
	lA49i.setText("A49:"+dform(A49i));
	lWound.setText("Wound:"+Wound);
	lDiff.setText("[ diff:"+dform(df)+" ]");
    }
    public String dform(double val) {
        return df1.format((float)val);
    }
    public String dform2(double val) {
        return df2.format((float)val);
    }
    //
    class myCanvas extends Canvas {
        private static final long serialVersionUID = 1L;
        public void update (Graphics g) {
            can.paint(g);
        }
        public void paint(Graphics g) {
            paintI();
            g.drawImage(ig, 0, 0, this);
        }
    }

    double DKEY;
    double Byh;
    double DY;
    double DB; // for beats
    int DOT;
    int DSF;

    public void paintI() {
	back_color(gi);

	DKEY = W/(double)KB;
	Byh = H/6.0;
	DOT = (DKEY < 2) ? 2 : (int)DKEY;
	DSF = DOT/2;
	DY = H2/MAXH; // cent
	DB = ABSsw ? H/MAXB : H2/MAXB; // beat

	if (!CALCsw) {
	    calc_(); // sinh,cosh,napi
	    calcBeat();
	}

	switch (GRAPH) {
        case INHA:
	    disp_aline(gi);
	    disp_log(gi);
	    disp_inha(gi);
	    disp_curve(gi);
	    break;
	case SINH:
	    disp_sinh(gi);
	    disp_se(gi);
	    disp_beat(gi);
	    break;
	case COSH:
	    disp_cosh(gi);
	    break;
	case NAPI:
	    disp_napier(gi);
	    disp_se(gi);
	    disp_beat(gi);
	    break;
	case BEATS:
	    disp_beats(gi);
	    disp_beat(gi);
	    break;
	case SMPL:
	    disp_sample(gi);
	    break;
	default:
	}

	if (thr != null)
	    disp_run(gi);

	write_tfdata();

	//color_test(gi);
    }
    public void back_color(Graphics g) {
        g.setColor(Color.black);
        g.fillRect(0, 0, W, H);
    }
    public void disp_run(Graphics g) {
	String st = "Now Runing.";
	int len = fm.stringWidth(st)/2;

	g.setColor(Color.cyan);
	g.drawString(st, W2-len, FNT*2);
    }
    //
    public void disp_curve(Graphics g) {
        for (int i = 0; i < KB; i++) {
	    cursor_color(g, i, Color.green);
	    g.fillOval((int)(i*DKEY), (int)calc_hsft(Cosh[i]), DOT, DOT);
	}

	write_label(diff_h(Cosh));
    }
    public void cursor_color(Graphics g, int k, Color c) {
	g.setColor((k == KEY) ? Color.orange : c);
    }
    //
    public void disp_inha(Graphics g) {
        for (int i = 0; i < KB; i++) {
	    cursor_color(g, i, Color.white);
            g.fillOval((int)(i*DKEY), (int)calc_hsft(MI.getInha(i)),
		       DOT, DOT);
	}
    }
    public double calc_h(double ih) { // log
	return H2-log_base(ih)*Byh;
    }
    public double calc_hsft(double ih) { // log-sft
	return H2-log_base(ih)*Byh-DSF;
    }
    //
    public void disp_cosh(Graphics g) {
	int val;
	boolean sw = (COSH_COUNTER > 0);
	disp_aline(g);
	if (sw) {
	    disp_log(g);
	} else {
	    disp_se(gi);
	    disp_cent_line(g);
	}

	for (int i = 0; i < KB; i++) {
	    val = (sw) ? (int)calc_h(Cosh[i]) : (int)calc_y(Cosh[i]);

	    cursor_color(g, i, Color.green);
	    g.fillOval((int)(DKEY*i), val, DOT, DOT);
	}
    }
    public double calc_y(double val) { // cent
	return H2-DY*val-DSF;
    }
    public double calc_yb(double val) { // for beats
	return ABSsw ? H-Math.abs(DB*val)-DSF : H2-DB*val-DSF;
    }
    //
    public void disp_sinh(Graphics g) {
	disp_aline(g);
	disp_cent_line(g);

	for (int i = 0; i < KB; i++) {
	    cursor_color(g, i, Color.yellow);
	    g.fillOval((int)(DKEY*i), (int)calc_y(Sinh[i]), DOT, DOT);
	}
    }
    //
    public void disp_napier(Graphics g) {
	disp_aline(g);
	disp_cent_line(g);

	for (int i = 0; i < KB; i++) {
	    cursor_color(g, i, Color.cyan);
	    g.fillOval((int)(DKEY*i), (int)calc_y(-Napi[i]), DOT, DOT);
	}
    }
    public void disp_se(Graphics g) {
	double val;

	for (int i = 0; i < KB; i++) {
	    val = Sinh[i]-Napi[i];//+Cosh[i];
	    cursor_color(g, i, Color.lightGray);
	    g.fillOval((int)(DKEY*i), (int)calc_y(val), DOT, DOT);
	}
    }
    //
    public void disp_beat(Graphics g) {
	Color bc = IN.getIColor(OCT2);
	int last = REVsw ? KB : KB-OCT2;
	int start = REVsw ? OCT2 : 0;
	int val;
	if (GRAPH != BEATS)
	    disp_y_line(g);

	for (int i = start; i < last; i++) {
	    val = (int)calc_yb(Beat[i]);
	    if (val < 0 || val > H)
		continue;
	    cursor_color(g, i, bc);
	    g.fillOval((int)(DKEY*i), val, DOT, DOT);
	}
    }
    //
    public void disp_beats(Graphics g) {
	disp_aline(g);
	disp_y_line(g);

	int num = chInterval.getSelectedIndex();
	disp_beats(g, num);
    }
    public void disp_beats(Graphics g, int num) {
	int max = REVsw ? KB : KB-IN.getIvalue(num);
	int start = REVsw ? IN.getIvalue(num) : 0;
	int val;

	calcBeats(num);

	Color bc = IN.getColor(num);
	for (int i = start; i < max; i++) {
	    val = (int)calc_yb(Beats[i]);
	    if (val < 0 || val > H)
		continue;
	    cursor_color(g, i, bc);
	    g.fillOval((int)(DKEY*i), val, DOT, DOT);
	}
    }
    public void disp_sample(Graphics g) {
	int di, val, sf = KB/2;
	if (!SMPLsw)
	    calc_sample(sf);

	disp_x_line(g);
	disp_cent_line(g);

	for (int i = 0; i < KB; i++) {
	    di = (int)(DKEY*i);
	    val = (int)calc_y(SampC[i]);
	    if (val > H || val < 0)
		continue;
	    cursor_color(g, i, Color.red);
	    g.fillOval(di, val, DOT, DOT);

	    val = (int)calc_y(SampS[i]);
	    if (val > H || val < 0)
		continue;
	    cursor_color(g, i, Color.white);
	    g.drawOval(di, val, DOT, DOT);
	}
    }
    public void calc_sample(int sf) {
	int ii;

	for (int i = 0; i < KB; i++) {
	    ii = i-sf;
	    SampC[i] = cosh(ii*SAMPLE);
	    SampS[i] = sinh(ii*SAMPLE);
	}

	SMPLsw = true;
    }
    public void disp_x_line(Graphics g) {
	int px, mx, xx;

	for (int i = 1; i < 5; i++) {
	    g.setColor(Color.blue);
	    xx = i*10;
	    px = (int)(W2+DKEY*xx+DSF);
	    g.drawLine(px, 0, px, H);
	    mx = (int)(W2-DKEY*xx+DSF);
	    g.drawLine(mx, 0, mx, H);

	    g.setColor(Color.lightGray);
	    g.drawString(dform2(xx), px, FNT);
	    g.drawString(dform2(-xx), mx, FNT);
	}

	g.setColor(Color.gray);
	g.drawLine(W2+DSF, 0, W2+DSF, H);
    }
    //
    public void disp_log(Graphics g) {
        double h0 = 0.001, hl;
        int hm;

        for (int i = 1; i < 6; i++) { // ʿ
            hl = h0*Math.pow(10, i);
            hm = (int)calc_h(hl);

            g.setColor(Color.blue);
            g.drawLine(0, hm, W, hm);

            g.setColor(Color.orange); // 
            g.drawString(""+(float)hl, 2, hm);
        }
    }
    public void disp_aline(Graphics g) {
        int wi, ii;

        for (int i = 0; i < KB; i += OCTAVE) {
	    wi = (int)(DKEY*i+DSF);

	    g.setColor(Color.blue);
	    g.drawLine(wi, 0, wi, H);

	    g.setColor(Color.red);
	    g.drawString(get_octave_name(i), wi, FNT);
	}
    }
    public String get_octave_name(int n) {
	return OCTAVE_NAME[n/OCTAVE];
    }
    public void disp_cent_line(Graphics g) {
	int py, my = 0;
	double yy, hi;
	double dy = MAXH/4;
	double dh = H2/MAXH;
	disp_0_line(g);

	for (int i = 1; i < 5; i++) {
	    hi = dy*i;
	    yy = hi*dh;
	    py = (int)(H2-yy);
	    my = (int)(H2+yy);

	    g.setColor(Color.blue);
	    g.drawLine(0, py, W, py);
	    g.drawLine(0, my, W, my);

	    g.setColor(Color.lightGray);
	    g.drawString(dform2(hi), 1, py);
	    g.drawString(dform2(-hi), 1, my);
	}
    }
    public void disp_y_line(Graphics g) {
	int py, my = 0;
	int sftp, sftm;
	int h2 = ABSsw ? H : H2;
	String stp, stm;
	double yy, hi;
	double dy = MAXB/4;
	double dh = h2/MAXB;
	disp_center_line(g);

	for (int i = 1; i < 5; i++) {
	    hi = dy*i;
	    yy = hi*dh;
	    py = (int)(h2-yy);
	    if (!ABSsw)
		my = (int)(H2+yy);

	    if (GRAPH == BEATS) {
		g.setColor(Color.blue);
		g.drawLine(0, py, W, py);
		if (!ABSsw)
		    g.drawLine(0, my, W, my);
	    }

	    stp = dform2(hi);
	    stm = dform2(-hi);
	    sftp = W-fm.stringWidth(stp);
	    sftm = W-fm.stringWidth(stm);
	    g.setColor(Color.pink);
	    g.drawString(stp, sftp, py);
	    if (!ABSsw)
		g.drawString(stm, sftm, my);
	}
    }
    public void disp_0_line(Graphics g) {
	g.setColor(Color.gray);
	g.drawLine(0, H2, W, H2);

	g.setColor(Color.lightGray);
	g.drawString("0", 1, H2);
    }
    public void disp_center_line(Graphics g) {
	if (!ABSsw && GRAPH == BEATS) {
	    g.setColor(Color.gray);
	    g.drawLine(0, H2, W, H2);
	}

	g.setColor(Color.pink);
	g.drawString("0", W-fm.stringWidth("0"), ABSsw ? H : H2);
    }
    public void color_test(Graphics g) {
	int num = IN.getLen();
	double dw = W/(double)num;

	for (int i = 0; i < num; i++) {
	    g.setColor(IN.getColor(i));
	    g.fillRect((int)(dw*i), H-FNT, (int)dw, FNT);

	    g.setColor(Color.white);
	    g.drawString(""+i, (int)(dw*i), H); 
	}
    }
    //
    public void calc_() {
	double df = A49i/get_cosh(48, Wound, Grade);

	for (int i = 0; i < KB; i++) {
	    Cosh[i] = get_cosh(i, Wound, Grade)*df;
	    Sinh[i] = get_sinh(i, Grade)*SHBE;
	    Napi[i] = get_e(i, Grade);
	}

	CALCsw = true;
    }
    public double log_base(double value) {
        return Math.log(value)/ML10;
    }
    public double get_cosh(int key, int ww, double grd) {
        return cosh((key-ww+1)*grd);
    }
    public double cosh(double x) {
        double me = Math.pow(Math.E, x);
        return (me+1/me)/2;
    }
    public double get_sinh(int key, double grd) {
        return sinh((key-KC)*grd);
    }
    public double sinh(double x) {
        double me = Math.pow(Math.E, x);
        return (me-1/me)/2;
    }
    public double get_e(int key, double gr) {
	double bb = -gr*0.6; //
	double a0 = calcE(48, bb)+A49i;

	return calcE(key, bb)-a0;
    }
    public double calcE(int key, double bb) {
	return BEAT*Math.pow(Math.E, bb*(key-KC));
    }
}
